package AlgLin;

/**
 * @author Hamadou BA
 * Depot GitLab : https://www-apps.univ-lehavre.fr/forge/bh243413/tp3-ps-inversion-de-matrices.git
 * 
 * La classe HilbertMatrice représente une matrice de Hilbert 
 * et permet d'effectuer des calculs matriciels.
 */


import java.util.Scanner;

public class HilbertMatrice extends Matrice {
	public HilbertMatrice(int ordre) {
		super(ordre, ordre);
		for (int i = 0; i < this.nbLigne(); i++) {
			for (int j = 0; j < this.nbColonne(); j++) {
				this.remplacecoef(i, j, 1.0 / (i + j + 1));
			}
		}
	}
	public static void main(String[] args) throws Exception {
	    Scanner entreeUtilisateur = new Scanner(System.in);
	    System.out.print("Veuillez saisir l'ordre de la matrice de Hilbert : ");
	    int taille = entreeUtilisateur.nextInt();

	    HilbertMatrice matriceHilbert = new HilbertMatrice(taille);
	    System.out.println(CYAN_COULEUR + "\nMatrice de Hilbert de dimension " + taille + " :" + REINITIALISER_COULEUR);
	    System.out.println(matriceHilbert.toString());
	    Matrice copieMatrice = new Matrice(matriceHilbert.nbLigne(), matriceHilbert.nbColonne());
	    copieMatrice.recopie(matriceHilbert);

	    Matrice matriceInverse = matriceHilbert.inverse();
	    System.out.println(CYAN_COULEUR + "Matrice inverse correspondante :" + REINITIALISER_COULEUR);
	    System.out.println(matriceInverse.toString());

	    System.out.println(CYAN_COULEUR + "\n-------- VALIDATION DES RÉSULTATS --------\n" + REINITIALISER_COULEUR);
	    Matrice produitMatrices = Matrice.produit(copieMatrice, matriceInverse);
	    Matrice matriceIdentite = Matrice.identite(taille);
	    System.out.println(CYAN_COULEUR + "Résultat du produit A * A^-1 :" + REINITIALISER_COULEUR);
	    System.out.println(produitMatrices.toString());

	    System.out.println(CYAN_COULEUR + "-------- ANALYSE DES ERREURS NUMÉRIQUES --------\n" + REINITIALISER_COULEUR);

	    Matrice ecart = Matrice.addition(produitMatrices, matriceIdentite.produit(-1));

	    double valeurNorme1 = ecart.norme_1();
	    System.out.println("Valeur de la norme 1 : " + valeurNorme1);
	    if (valeurNorme1 < Matrice.EPSILON) {
	        System.out.println(JAUNE_COULEUR + "Norme 1 proche de zéro (erreur faible)\n" + REINITIALISER_COULEUR);
	    } else {
	        System.out.println(MAGENTA_COULEUR + "Norme 1 significative (erreur notable)\n" + REINITIALISER_COULEUR);
	    }

	    double valeurNormeInf = ecart.norme_inf();
	    System.out.println("Valeur de la norme infinie : " + valeurNormeInf);
	    if (valeurNormeInf < Matrice.EPSILON) {
	        System.out.println(JAUNE_COULEUR + "Norme infinie proche de zéro (précision acceptable)\n" + REINITIALISER_COULEUR);
	    } else {
	        System.out.println(MAGENTA_COULEUR + "Norme infinie non négligeable (erreur présente)\n" + REINITIALISER_COULEUR);
	    }

	    double conditionnementNorme1 = matriceInverse.cond_1();
	    double conditionnementNormeInf = matriceInverse.cond_inf();
	    System.out.println("Indice de conditionnement (norme 1) : " + conditionnementNorme1);
	    System.out.println("Indice de conditionnement (norme infinie) : " + conditionnementNormeInf);

	    entreeUtilisateur.close();
	}

}
