package AlgLin;

/**
 * @author Hamadou BA
 * Depot GitLab : https://www-apps.univ-lehavre.fr/forge/bh243413/tp3-ps-inversion-de-matrices.git
 * 
 * La classe Vecteur represente une matrice
 * permettant de manipuler et effectuer des calculs sur des matrices
 */


import java.io.File;
import java.io.FileNotFoundException;
import java.util.Scanner;

public class Matrice {
	protected double coefficient[][];
	public final static double EPSILON = 1e-6;
	public static final String REINITIALISER_COULEUR = "\u001B[0m";
	public static final String JAUNE_COULEUR = "\u001B[33m";
	public static final String CYAN_COULEUR = "\u001B[36m";
	public static final String MAGENTA_COULEUR = "\u001B[35m";
	public static final String ORANGE_COULEUR = "\u001B[38;5;214m";
	
	Matrice(int nbligne, int nbcolonne) {
		this.coefficient = new double[nbligne][nbcolonne];
	}

	Matrice(double[][] tableau) {
		coefficient = tableau;
	}

	Matrice(String fichier) {
		try {
			Scanner sc = new Scanner(new File(fichier));
			int ligne = sc.nextInt();
			int colonne = sc.nextInt();
			this.coefficient = new double[ligne][colonne];
			for (int i = 0; i < ligne; i++)
				for (int j = 0; j < colonne; j++)
					this.coefficient[i][j] = sc.nextDouble();
			sc.close();

		} catch (FileNotFoundException e) {
			System.out.println("Fichier absent");
		}
	}

	public void recopie(Matrice arecopier) {
		int ligne, colonne;
		ligne = arecopier.nbLigne();
		colonne = arecopier.nbColonne();
		this.coefficient = new double[ligne][colonne];
		for (int i = 0; i < ligne; i++)
			for (int j = 0; j < colonne; j++)
				this.coefficient[i][j] = arecopier.coefficient[i][j];
	}

	public int nbLigne() {
		return this.coefficient.length;
	}

	public int nbColonne() {
		return this.coefficient[0].length;
	}

	public double getCoeff(int ligne, int colonne) {
		return this.coefficient[ligne][colonne];
	}

	public void remplacecoef(int ligne, int colonne, double value) {
		this.coefficient[ligne][colonne] = value;
	}

	public String toString() {
		int ligne = this.nbLigne();
		int colonne = this.nbColonne();
		String matr = "";
		for (int i = 0; i < ligne; i++) {
			for (int j = 0; j < colonne; j++) {
				if (j == 0) {
					matr += this.getCoeff(i, j);
				} else {
					matr += " " + this.getCoeff(i, j);
				}
			}
			matr += "\n";
		}
		return matr;
	}

	public Matrice produit(double scalaire) {
		int ligne = this.nbLigne();
		int colonne = this.nbColonne();
		for (int i = 0; i < ligne; i++)
			for (int j = 0; j < colonne; j++)
				this.coefficient[i][j] *= scalaire;
		return this;
	}

	static Matrice addition(Matrice a, Matrice b) {
		int ligne = a.nbLigne();
		int colonne = a.nbColonne();
		Matrice mat = new Matrice(ligne, colonne);
		for (int i = 0; i < ligne; i++)
			for (int j = 0; j < colonne; j++)
				mat.coefficient[i][j] = a.coefficient[i][j] + b.coefficient[i][j];
		return mat;
	}

	static Matrice verif_addition(Matrice a, Matrice b) throws Exception {
		if ((a.nbLigne() == b.nbLigne()) && (a.nbColonne() == b.nbColonne())) {
			int ligne = a.nbLigne();
			int colonne = a.nbColonne();
			Matrice mat = new Matrice(ligne, colonne);
			for (int i = 0; i < ligne; i++)
				for (int j = 0; j < colonne; j++)
					mat.coefficient[i][j] = a.coefficient[i][j] + b.coefficient[i][j];
			return mat;
		} else {
			throw new Exception("Les deux matrices n'ont pas les mêmes dimensions !!!");
		}
	}

	static Matrice produit(Matrice a, Matrice b) {
		int ligne, colonne;
		ligne = a.nbLigne();
		colonne = b.nbColonne();
		Matrice mat = new Matrice(ligne, colonne);
		for (int i = 0; i < ligne; i++)
			for (int j = 0; j < colonne; j++) {
				mat.coefficient[i][j] = 0;
				for (int k = 0; k < a.nbColonne(); k++)
					mat.coefficient[i][j] += a.coefficient[i][k] * b.coefficient[k][j];
			}
		return mat;
	}

	static Matrice verif_produit(Matrice a, Matrice b) throws IrregularSysLinException {
		int ligne = 0;
		int colonne = 0;
		if (a.nbColonne() == b.nbLigne()) {
			ligne = a.nbLigne();
			colonne = b.nbColonne();
		} else {
			throw new IrregularSysLinException("Dimensions des matrices à multiplier incorrectes");
		}

		Matrice mat = new Matrice(ligne, colonne);
		for (int i = 0; i < ligne; i++)
			for (int j = 0; j < colonne; j++) {
				mat.coefficient[i][j] = 0;
				for (int k = 0; k < a.nbColonne(); k++)
					mat.coefficient[i][j] += a.coefficient[i][k] * b.coefficient[k][j];
			}
		return mat;
	}


	public static Matrice identite(int n) {
		Matrice identite = new Matrice(n, n);
		for (int i = 0; i < n; i++) {
			for (int j = 0; j < n; j++) {
				if (i == j) {
					identite.remplacecoef(i, j, 1);
				} else {
					identite.remplacecoef(i, j, 0);
				}
			}
		}
		return identite;
	}


	public Matrice inverse() throws IrregularSysLinException {
		if (this.nbLigne() != this.nbColonne()) {
			throw new IrregularSysLinException("La matrice n'est pas carrée.");
		}
		int n = this.nbLigne();
		Matrice inverse = new Matrice(n, n);

		Helder helder = new Helder(this, new Vecteur(n));
		helder.factorLDR();

		for (int i = 0; i < n; i++) {
			Vecteur secondMembre = new Vecteur(n);
			for (int j = 0; j < n; j++) {
				secondMembre.remplacecoef(j, (j == i) ? 1.0 : 0.0);
			}

			helder.setSecondMembre(secondMembre);
			Vecteur solution = helder.resolutionPartielle();

			for (int j = 0; j < n; j++) {
				inverse.remplacecoef(j, i, solution.getCoeff(j));
			}
		}
		return inverse;
	}

	public double norme_1() {
		double norme = 0.0;
		for (int j = 0; j < this.nbColonne(); j++) {
			double sum = 0.0;
			for (int i = 0; i < this.nbLigne(); i++) {
				sum += Math.abs(this.coefficient[i][j]);
			}
			norme = Math.max(norme, sum);
		}
		return norme;
	}


	public double norme_inf() {
		double norme = 0.0;
		for (int i = 0; i < this.nbLigne(); i++) {
			double sumL = 0.0;
			for (int j = 0; j < this.nbColonne(); j++) {
				sumL += Math.abs(this.coefficient[i][j]);
			}
			norme = Math.max(norme, sumL);
		}
		return norme;
	}


	public double cond_1() throws Exception {
		Matrice inverse = this.inverse();
		double norme1 = this.norme_1();
		double norme1_inverse = inverse.norme_1();
		if (norme1_inverse == 0) {
			throw new Exception("La norme 1 de la matrice inverse est nulle.");
		}
		return norme1 * norme1_inverse;
	}

	
	public double cond_inf() throws Exception {
		Matrice inverse = this.inverse();
		double normeInf = this.norme_inf();
		double normeInfInverse = inverse.norme_inf();
		if (normeInfInverse == 0) {
			throw new Exception("La norme infinie de la matrice inverse est nulle.");
		}
		return normeInf * normeInfInverse;
	}

	public static void main(String[] args) throws Exception {
	    double tableauMatrice[][] = { { 2, 1 }, { 0, 1 } };
	    Matrice matriceA = new Matrice(tableauMatrice);
	    System.out.println("Création d'une matrice à partir d'un tableau :\n" + matriceA);

	    Matrice matriceB = new Matrice("./resources/matrice1.txt");
	    System.out.println("Chargement d'une matrice depuis un fichier :\n" + matriceB);

	    Matrice matriceC = new Matrice(2, 2);
	    matriceC.recopie(matriceB);
	    System.out.println("Copie de la matrice B :\n" + matriceC);

	    System.out.println("Dimensions de la matrice C : " + matriceC.nbLigne() + " x " + matriceC.nbColonne());
	    System.out.println("Valeur du coefficient (2,2) dans la matrice B : " + matriceB.getCoeff(1, 1));
	    System.out.println("Mise à jour de ce coefficient à 8");
	    matriceB.remplacecoef(1, 1, 8);
	    System.out.println("Vérification de la modification : " + matriceB.getCoeff(1, 1));

	    System.out.println("\nAddition de deux matrices :\nMatrice 1 :\n" + matriceA + "Matrice 2 :\n" + matriceB + "Résultat :\n" + Matrice.addition(matriceA, matriceB));
	    System.out.println("Multiplication de deux matrices :\nMatrice 1 :\n" + matriceA + "Matrice 2 :\n" + matriceB + "Résultat :\n" + Matrice.produit(matriceA, matriceB));

	    System.out.println(CYAN_COULEUR + "\n--------------- INVERSION DE MATRICE ---------------\n" + REINITIALISER_COULEUR);
	    Matrice matriceOriginale = new Matrice("./resources/matrice1.txt");
	    System.out.println("Matrice initiale :\n" + matriceOriginale);

	    Matrice copieMatrice = new Matrice(matriceOriginale.nbLigne(), matriceOriginale.nbColonne());
	    copieMatrice.recopie(matriceOriginale);

	    Matrice matriceInverse = matriceOriginale.inverse();
	    System.out.println("Matrice inversée :\n" + matriceInverse);

	    System.out.println(CYAN_COULEUR + "\n--------------- VÉRIFICATION ---------------\n" + REINITIALISER_COULEUR);
	    Matrice produitMatrices = Matrice.produit(copieMatrice, matriceInverse);
	    Matrice matriceIdentite = Matrice.identite(matriceOriginale.nbLigne());
	    System.out.println("Produit A * A^-1 : \n" + produitMatrices);

	    Matrice ecart = Matrice.addition(produitMatrices, matriceIdentite.produit(-1));
	    System.out.println(CYAN_COULEUR + "\n--------------- ANALYSE DES NORMES ---------------\n" + REINITIALISER_COULEUR);
	    
	    double valeurNorme1 = ecart.norme_1();
	    System.out.println("Valeur de la norme 1 : " + valeurNorme1);
	    System.out.println(valeurNorme1 < Matrice.EPSILON ? JAUNE_COULEUR + "Norme 1 proche de zéro (erreur négligeable)" + REINITIALISER_COULEUR : MAGENTA_COULEUR + "Norme 1 non nulle (écart significatif)" + REINITIALISER_COULEUR);
	    
	    double valeurNormeInf = ecart.norme_inf();
	    System.out.println("Valeur de la norme infinie : " + valeurNormeInf);
	    System.out.println(valeurNormeInf < Matrice.EPSILON ? JAUNE_COULEUR + "Norme infinie proche de zéro (précision correcte)" + REINITIALISER_COULEUR : MAGENTA_COULEUR + "Norme infinie non négligeable (erreur notable)" + REINITIALISER_COULEUR);

	    System.out.println(CYAN_COULEUR + "\n--------------- CONDITIONNEMENT ---------------\n" + REINITIALISER_COULEUR);
	    double conditionnementNorme1 = matriceInverse.cond_1();
	    double conditionnementNormeInf = matriceInverse.cond_inf();
	    System.out.println("Indice de conditionnement (norme 1) : " + conditionnementNorme1);
	    System.out.println("Indice de conditionnement (norme infinie) : " + conditionnementNormeInf);
	}

}
