package AlgLin;


/**
 * @author Hamadou BA  
 * Depot GitLab : https://www-apps.univ-lehavre.fr/forge/bh243413/tp3-ps-inversion-de-matrices.git
 *  
 * Classe representant un système lineaire avec une matrice triangulaire inferieure.
 */


public class SystTriangInf extends SysLin {

	public SystTriangInf(Matrice m, Vecteur v) throws IrregularSysLinException {
		super(m, v);
	}


	@Override
	public Vecteur resolution() throws IrregularSysLinException {
		Matrice matrice = getMatriceSystem();
		Vecteur res = new Vecteur(matrice.nbLigne());

		for (int i = 0; i < matrice.nbLigne(); i++) {
			double sum = 0.0;
			for (int j = 0; j < i; j++) {
				sum += matrice.getCoeff(i, j) * res.getCoeff(j);
			}
			if (Math.abs(matrice.getCoeff(i, i)) < Matrice.EPSILON)
				throw new IrregularSysLinException("Le système diagonal est irrégulier.");

			res.remplacecoef(i, (secondMembre.getCoeff(i) - sum) / matrice.getCoeff(i, i));
		}
		return res;
	}

	public static void main(String... args) throws IrregularSysLinException {
		System.out.println("*** Test de la classe SystTriangInf ***");
		double mat[][] = { { 2, 0, 0 }, { 1, 2, 0 }, { 3, 4, 5 } };
		Matrice matriceA = new Matrice(mat);
		Vecteur vecteurB = new Vecteur(new double[] { 1, 2, 3 });
		SysLin sys = new SystTriangInf(matriceA, vecteurB);
		Vecteur solutionX = sys.resolution();
		System.out.println("x = " + solutionX);

		Matrice _b = new Matrice(vecteurB.getTaille(), 1);
		for (int i = 0; i < vecteurB.getTaille(); i++) {
			_b.remplacecoef(i, 0, vecteurB.getCoeff(i));
			_b.remplacecoef(i, 0, -_b.getCoeff(i, 0));
		}

		Matrice Ax = Matrice.produit(matriceA, solutionX); // Ax
		System.out.println("Le résultat Ax : \n" + Ax);

		Matrice Ax_b = Matrice.addition(Ax, _b); // Ax - b
		System.out.println("Le résultat Ax - b : " + Ax_b);
		Vecteur norme = new Vecteur(Ax_b.nbLigne());
		for (int i = 0; i < Ax_b.nbLigne(); i++) {
			norme.remplacecoef(i, Ax_b.getCoeff(i, 0));
		}

		double normeL1 = Vecteur.normeL1(norme);
		System.out.println("Norme L1 : " + normeL1);
		if (normeL1 <= 0.0 || normeL1 > Matrice.EPSILON) {
			System.out.println("=> La norme du vecteur est nulle ou très petite");
		}
		double normeL2 = Vecteur.normeL2(norme);
		System.out.println("Norme L2 : " + normeL2);
		if (normeL2 <= 0.0 || normeL2 > Matrice.EPSILON) {
			System.out.println("=> La norme du vecteur est nulle ou très petite");
		}
		double normeLinfini = Vecteur.normeLinfini(norme);
		System.out.println("Norme Linfini : " + normeLinfini);
		if (normeLinfini <= 0.0 || normeLinfini > Matrice.EPSILON) {
			System.out.println("=> La norme du vecteur est nulle ou très petite");
		}
	}

}
